@@ -5,7 +5,7 @@
 # Date:   December 12, 2008                                                    #
 #                                                                              #
 ################################################################################
-get.knn<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR"))
+get.knn<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR", "brute"))
 {
   algorithm<- match.arg(algorithm);   
   if(storage.mode(data)=="integer") storage.mode(data)<- "double";
@@ -19,7 +19,8 @@ get.knn<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR
               cover_tree = "get_KNN_cover",
               kd_tree= "get_KNN_kd",
               VR = "get_KNN_VR",
-              CR = "get_KNN_CR"
+              CR = "get_KNN_CR",
+              brute = "get_KNN_brute"
   ); 
   knnres<- .C(Cname, t(data), as.integer(k), d, n, nn.index = integer(n*k), nn.dist = double(n*k), DUP=FALSE);
                                            
@@ -33,7 +34,7 @@ get.knn<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR
     
   return(list(nn.index=nn.index, nn.dist=nn.dist));    
 }
-get.knnx<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR"))
+get.knnx<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR", "brute"))
 {
   #k neearest neighbor Euclidean distances
   algorithm<- match.arg(algorithm);	  
@@ -53,7 +54,8 @@ get.knnx<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "
                 cover_tree = "get_KNNX_cover",
                 kd_tree= "get_KNNX_kd",                 
                 VR = "get_KNNX_VR",
-                CR = "get_KNNX_CR"
+                CR = "get_KNNX_CR",
+                brute = "get_KNNX_brute"
   ); 
   knnres<- .C(Cname, t(data), t(query), as.integer(k), d, n, m, nn.index = integer(m*k), nn.dist = double(m*k), DUP=FALSE);
                                              
@@ -67,19 +69,19 @@ get.knnx<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "
 
   return(list(nn.index=nn.index, nn.dist=nn.dist));
 }
-knnx.dist<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR"))
+knn.index<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR", "brute"))
+{
+  get.knn(data, k, algorithm )$nn.index;
+}
+knn.dist<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR", "brute"))
+{
+  get.knn(data, k, algorithm )$nn.dist;
+}
+knnx.dist<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR", "brute"))
 {
   get.knnx(data, query, k, algorithm )$nn.dist  
 }
-knnx.index<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR"))
+knnx.index<- function (data, query, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR", "brute"))
 {
   get.knnx(data, query, k, algorithm )$nn.index;
 }
-knn.index<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR"))
-{
-  get.knn(data, k, algorithm )$nn.index;  
-}
-knn.dist<- function (data, k = 10, algorithm=c("cover_tree", "kd_tree", "VR", "CR"))
-{
-  get.knn(data, k, algorithm )$nn.dist; 
-}
