@@ -20,6 +20,108 @@ void Rvector2ANNarray(ANNpointArray data_pts,
 } 
 
 extern "C" {
+
+void get_KNN_brute(double *data, const int *k, const int *dim, const int *n_pts, int *nn_idx, double *nn_dist)
+{
+	const int	d=*dim;		// Actual Dimension
+	const int	n=*n_pts;	// Number of Data points
+	const int	K = *k+1;	// Max. num of NN. first one  (itself) will discarded.
+	const double		error_bound = 0.0;
+	register int ptr = 0;
+
+	ANNidxArray		index = new ANNidx[K];		// Near neighbor indices
+	ANNdistArray	dist = new ANNdist[K];		// Near neighbor nn_dist
+    ANNpointArray	data_pts = new ANNpoint[n];
+	if(data_pts==NULL) error("Cannot allocate memroy for data matrix!\n");
+
+	Rvector2ANNarray(data_pts, data, n, d);
+
+	ANNbruteForce	*kd_tree = new ANNbruteForce(data_pts, n, d);
+
+	for(int i = 0; i < n; i++)	// read query points
+	{
+	    kd_tree->annkSearch(	// search
+			data_pts[i],		  // query point
+			K,				      // number of near neighbors
+			index,				  // nearest neighbors index
+			dist,				  // squared distance
+			error_bound);		  // error bound
+
+	    for (int j = 1; j < K; j++)      //return result row by row
+	    {
+	    	nn_dist[ptr] = sqrt(dist[j]);	// unsquare distance
+            nn_idx[ptr]  = index[j] + 1;	// put indexes in returned array
+			ptr++;
+	    } // end inner for
+	} // end for
+
+	delete[] index;
+	delete[] dist;
+	delete kd_tree;
+	delete[] data_pts;
+	annClose();
+
+}//end get_KNN_brute
+void get_KNNX_brute(double *data, double* query,
+                 int *k, int *dim,
+                 int *n_pts, int *m_pts,
+                 int *nn_idx, double *nn_dist)
+{
+	const int				d=*dim;		// Actual Dimension
+	const int				n=*n_pts;	// Number of Base Data points
+	const int				m=*m_pts;	// Number of Query Data points
+	const int				K = *k;		// Max. num of NN
+	const double		error_bound = 0.0;
+
+	ANNidxArray		index = new ANNidx[K];		// Near neighbor indices
+	ANNdistArray	dist = new ANNdist[K];		// Near neighbor nn_dist
+//	ANNpointArray	data_pts  = annAllocPts(n, d);	// Base Data points
+//	ANNpointArray	query_pts  = annAllocPts(m, d);	// Query Data points
+    ANNpointArray	data_pts  = new ANNpoint[n];
+    ANNpointArray	query_pts  = new ANNpoint[m];
+
+	if(data_pts==NULL) error("Cannot allocate memroy for data matrix!\n");
+	if(query_pts==NULL) error("Cannot allocate memroy for query data matrix!\n");
+
+	//copy data into  matrix from a vector column by column
+	Rvector2ANNarray(data_pts, data, n, d);
+ 	Rvector2ANNarray(query_pts, query, m, d);
+
+	ANNbruteForce	*kd_tree	= new ANNbruteForce(data_pts, n, d);
+
+ 	register int ptr = 0;
+	for(int i = 0; i < m; i++)	// read query points
+	{
+	    kd_tree->annkSearch(	// search
+			query_pts[i],		// query point
+			K,				// number of near neighbors
+			index,				// nearest neighbors index
+			dist,				// squared distance
+			error_bound);		// error bound
+
+	    for (int j = 0; j < K; j++)      //return result row by row
+	    {
+	    	nn_dist[ptr] = sqrt(dist[j]);	// unsquare distance
+            nn_idx[ptr]  = index[j] + 1;	// put indexes in returned array
+			ptr++;
+	    } // end inner for
+	} // end for
+
+	delete[] index;
+	delete[] dist;
+//	delete kd_tree;
+
+//	annDeallocPts(data_pts);
+//	annDeallocPts(query_pts);
+	delete[] data_pts;
+	delete[] query_pts;
+	delete kd_tree;
+
+
+	annClose();
+
+}//end get_KNNX_brute
+
 void get_KNN_kd(double *data, const int *k, const int *dim, const int *n_pts, int *nn_idx, double *nn_dist)
 {
 	const int	d=*dim;		// Actual Dimension
@@ -30,7 +132,7 @@ void get_KNN_kd(double *data, const int *k, const int *dim, const int *n_pts, in
 	
 	ANNidxArray		index = new ANNidx[K];		// Near neighbor indices
 	ANNdistArray	dist = new ANNdist[K];		// Near neighbor nn_dist
-  ANNpointArray	data_pts = new ANNpoint[n];		
+    ANNpointArray	data_pts = new ANNpoint[n];
 	if(data_pts==NULL) error("Cannot allocate memroy for data matrix!\n");
 	
 	Rvector2ANNarray(data_pts, data, n, d);
@@ -77,8 +179,8 @@ void get_KNNX_kd(double *data, double* query,
 	ANNdistArray	dist = new ANNdist[K];		// Near neighbor nn_dist		
 //	ANNpointArray	data_pts  = annAllocPts(n, d);	// Base Data points
 //	ANNpointArray	query_pts  = annAllocPts(m, d);	// Query Data points
-  ANNpointArray	data_pts  = new ANNpoint[n];			
-  ANNpointArray	query_pts  = new ANNpoint[m];			
+    ANNpointArray	data_pts  = new ANNpoint[n];
+    ANNpointArray	query_pts  = new ANNpoint[m];
 
 	if(data_pts==NULL) error("Cannot allocate memroy for data matrix!\n");
 	if(query_pts==NULL) error("Cannot allocate memroy for query data matrix!\n");
@@ -111,7 +213,7 @@ void get_KNNX_kd(double *data, double* query,
 	delete[] dist;	
 //	delete kd_tree;
 
-	//	annDeallocPts(data_pts);
+//	annDeallocPts(data_pts);
 //	annDeallocPts(query_pts);
 	delete[] data_pts;
 	delete[] query_pts;
@@ -122,8 +224,55 @@ void get_KNNX_kd(double *data, double* query,
 	
 }//end get_KNNX_kd
 
+void KNN_MLD_brute(double *data, const int *k, const int *dim,
+                const int *n_pts, double *MLD)
+//mean of log distances
+{
+ 	//For large dataset, use less memory than returning distances.
+	const int				d=*dim;		// Actual Dimension
+	const int				n=*n_pts;	// Number of Data points
+	const int				K = *k+1;		// Max. num of NN. first one  (itself) will discarded.
+	const double		error_bound = 0.0;
 
-void KNN_MLD(double *data, const int *k, const int *dim,
+	ANNidxArray		index = new ANNidx[K];		// Near neighbor indices
+	ANNdistArray	dist = new ANNdist[K];		// Near neighbor nn_dist
+    ANNpointArray	data_pts  = new ANNpoint[n];
+	if(data_pts==NULL) error("Cannot allocate memroy for data matrix!\n");
+
+	//copy data into n x d matrix from n*d vector column by column
+	Rvector2ANNarray(data_pts, data, n, d);
+
+	ANNbruteForce	*kd_tree = new ANNbruteForce(data_pts, n, d);
+
+	for(int i = 0; i < n; i++)	// read query points
+	{
+	    kd_tree->annkSearch(	// search
+			data_pts[i],		// query point
+			K,				     // number of near neighbors
+			index,				// nearest neighbors index
+			dist,				// squared distance
+			error_bound);		// error bound
+
+	    for (int j = 0; j < K-1; j++)
+	    {
+	    	MLD[j] += log(dist[j+1]);  //discard first one (0 distance).
+	    } // end inner for
+	} // end for
+
+	for (int j = 0; j < K-1; j++)
+	{
+	  	MLD[j] /= 2*n; //colMeans(log(dnn))
+	}
+
+	delete[] index;
+	delete[] dist;
+	delete kd_tree;
+    delete[] data_pts;
+	annClose();
+
+}//end KNN_MLD_brute
+
+void KNN_MLD_kd(double *data, const int *k, const int *dim,
                 const int *n_pts, double *MLD)
 //mean of log distances                
 {
@@ -135,7 +284,7 @@ void KNN_MLD(double *data, const int *k, const int *dim,
 	
 	ANNidxArray		index = new ANNidx[K];		// Near neighbor indices
 	ANNdistArray	dist = new ANNdist[K];		// Near neighbor nn_dist	
-  ANNpointArray	data_pts  = new ANNpoint[n];			
+    ANNpointArray	data_pts  = new ANNpoint[n];
 	if(data_pts==NULL) error("Cannot allocate memroy for data matrix!\n");
 			
 	//copy data into n x d matrix from n*d vector column by column
@@ -166,10 +315,10 @@ void KNN_MLD(double *data, const int *k, const int *dim,
 	delete[] index;
 	delete[] dist;	
 	delete kd_tree;
-  delete[] data_pts;
+    delete[] data_pts;
 	annClose();
 	
-}//end KNN_MLD
+}//end KNN_MLD_kd
 
 
 void KL_divergence(double *X, double* Y, const int *k, const int *dim,
@@ -182,11 +331,11 @@ void KL_divergence(double *X, double* Y, const int *k, const int *dim,
 	const int			K = *k;		// Max. num of NN	
 	const double	error_bound = 0.0;
 	
-  double	*XX_dist= new double[K];
+    double	*XX_dist= new double[K];
   
 	double	*XY_dist= new double[K];
-  ANNpointArray	X_pts = new ANNpoint[n];		// X points		
-  ANNpointArray	Y_pts = new ANNpoint[m];		// Y points	
+    ANNpointArray	X_pts = new ANNpoint[n];		// X points
+    ANNpointArray	Y_pts = new ANNpoint[m];		// Y points
 	if(X_pts==NULL) error("Cannot allocate memroy for X vector!\n");
 	if(Y_pts==NULL) error("Cannot allocate memroy for Y vector!\n");
 		
@@ -206,9 +355,9 @@ void KL_divergence(double *X, double* Y, const int *k, const int *dim,
 	    for (int j = 0; j < K; j++)	 XY_dist[j] += log(dist[j]);	
 	}
 	
-  delete Y_tree;
+    delete Y_tree;
 //	annDeallocPts(Y_pts); // release memory ASAP
-  delete[] Y_pts;
+    delete[] Y_pts;
 	ANNkd_tree *X_tree = new ANNkd_tree(X_pts, n, d);	
 
 	for (int j = 0; j < K; j++)	 XX_dist[j] =  0.0;	   
@@ -217,7 +366,7 @@ void KL_divergence(double *X, double* Y, const int *k, const int *dim,
 	    for (int j = 0; j < K; j++)	 XX_dist[j] += log(dist[j+1]);		    
 	} 
 
-  delete[] index;
+    delete[] index;
 	delete[] dist;
  	delete X_tree;
 	delete[] X_pts;
@@ -256,8 +405,8 @@ void KL_dist(double *X, double* Y, const int *k, const int *dim,
 	}
  	ANNidxArray		index = new ANNidx[K+1];		// Near neighbor indices
 	ANNdistArray	dist = new ANNdist[K+1];		// Near neighbor nn_dist
-  ANNpointArray	X_pts = new ANNpoint[n];		// X points	
-  ANNpointArray	Y_pts = new ANNpoint[m];	//Y points		
+    ANNpointArray	X_pts = new ANNpoint[n];		// X points
+    ANNpointArray	Y_pts = new ANNpoint[m];	//Y points
 	
 	if(X_pts==NULL) error("Cannot allocate memroy for X vector!\n");
 	if(Y_pts==NULL) error("Cannot allocate memroy for Y vector!\n");
@@ -300,10 +449,10 @@ void KL_dist(double *X, double* Y, const int *k, const int *dim,
 
 	delete[] index;
 	delete[] dist;
-  delete Y_tree;
+    delete Y_tree;
 
-  delete[] X_pts;
-  delete[] Y_pts;
+    delete[] X_pts;
+    delete[] Y_pts;
  	annClose();	
 
 	for (int j = 0; j < K; j++)	
